%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ys,check]=BGG_steadystate(ys,exe)
% compute s.s.
global M_ options_
check = 0;

%the following statements ensure that the variables are not interpreted by
%MATLAB as functions...
mu=1; gamma=1; alpha=1; delta=1; beta=1; nu=1; theta=1; eps=1;
% the correct values (as in themod file) will be loaded below

% Here we load the values of the deep parameters in a loop.
Np = M_.param_nbr;                                            
for I = 1:Np
    paramname = deblank(M_.param_names(I,:));
    eval([ paramname ' = M_.params(' int2str(I) ');']);
end

%% Find the cutoff value for omega
format long
flag = 0;
omegabar_guess = 0.54; % 0.53057;
omb = omegabar_guess;
incr = 0;
while flag ==0
    omb     = omb + incr;
    S       = S_ss;         % Fixed to 0.26
    infl    = infl_ss;      % fixed to 4 percent
    rn      = infl/beta-1;  % from (3)
    q       = 1;            % from (12)
    F       = (normcdf(((log(omb) + S^2 / 2) / S)));
    Fp1     = F;
    G       = (normcdf((log(omb) + S^2 / 2) / S - S));
    Gp1     = G;
    GAMMA   = omb*(1-F)+G;
    GAMMAp1 = GAMMA;
    dFp1    = normpdf((log(omb) + S^2 / 2) / S) / omb / S;
    % Compute rk using omegabar_guess and rn from efficiency condition
    % associated with the contract (rn, rk, omb) in (6)
    rk      = (1+rn)/( GAMMA-mu*G + (1 - F -mu*omb*dFp1)*((1-GAMMA)/(1-F)) ) - 1 ;
    % Compute nw/K from Law of motion of net worth (9)
    nk1     = (gamma*(1-GAMMA)*q*(1+rk))/infl;
    % Compute nw/K from Zero profit condition (10)
    nk2     = 1 - ((1+rk)/(1+rn))*(GAMMA-mu*G);
    % In steady state these ratios have to be equalized: the loop looks for
    % the omb that makes nk1=nk2, gien a value for rk and rn
    
    if abs(nk1 - nk2)< 10^-8
        disp('---------------------------------------------------------');
        disp('Convergence! Omega bar has been found');
        disp('---------------------------------------------------------');
        flag = 1;
    else
        if nk1>nk2
            incr = -(nk1 - nk2); % make the increase proportional to the distance to the solution
        else
            incr = -(nk1 - nk2); % make the increase proportional to the distance to the solution
        end
    end
end

%% Find the remaining ss values
yk = (q*(1 + rk) ) / infl;
efp = (1-Fp1)/( (1-Fp1)*(GAMMAp1-mu*Gp1)...  %(6)
    + (1-GAMMAp1)*(1-Fp1-mu*omb*dFp1) );
mc = (eps -1)/eps;                          %(10)
A  = 0; W = 0; X=1;                %(14)
z  = (q*(1 + rk))/infl + delta -1;          %(7)
N  = 0.3;



Y_K = z/(mc*alpha);                         %(5)&(C)
K_N = Y_K^(1/(alpha-1));                    %(5)&(C)
K = K_N*N;                                  %(5)&(C)
Y = exp(A)*(K^alpha)*(N^(1-alpha));              %(Y) 
I_K = delta;                                %(2)
C_K = Y_K - I_K - mu*G*q*(1+rk);            %(1)and(d)
I  = K * delta ;                            %(2)
d  = mu * G  * q*(1 + rk) * K;              %(a)
Ce = (1-gamma)*(1-GAMMA)*yk*K;          %(13)
C  = Y - (Ce + I + d);                      %(1)
Yk = z/mc;                                  %(5)
Yn = exp(A)*(1-alpha)*(K^alpha)*(N^(-alpha));    %(C)
nw = (gamma*(1-GAMMA)*q*(1+rk)*K)/infl;     %(7) or nw = K*(1 - ((1+rk)/(1+rn))*(GAMMA-mu*G)); %(6)
GDP = C + Ce + I;                           %(d)

Uc      = (C - tau*N^nu)^(-ra);             %(b)
Ucp1    = Uc;                               %(b)
Un      = -tau*nu*N^(nu-1)*(C - tau*N^nu)^(-ra); %(b)

dflt = F;
dstr = d;
    L = (q*K)/nw;
    B = q*K - nw;
    Rn = 1 + rn;
            rp = (efp/omb)*(1-(nw/(q*K)));
            RL = Rn+(rp-1);
    Rk = 1+rk;
    Rr = 1 + (Rn-infl);
    wage = mc*Yn;
    WW = exp(W);

% To get the correct calibration given the value of N above
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%
% GHH:      
set_tau_GHH_to = (mc*Yn)/((1+nu)*(N^nu))
% NonSep:   
set_tau_NonSep_to = (mc*Yn*(1-N))/C
% Sep:      
set_tau_Sep_to = C^(-ra)*mc*Yn/(1-N^nu)
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%

%%
% Define the steady state values of the endogenous variables of the model.
Ne = M_.orig_endo_nbr;
ys = zeros(Ne,1);
endoleadcount = 2;
nonauxcount = 0;
for indexvar = 1:Ne
    varname = deblank(M_.endo_names(indexvar,:));
        eval(['ys(' int2str(indexvar) ') = ' varname ';']);
end

1+1;