%% Preliminaries
% set(0,'DefaultTextFontName','Palatino')
% set(0,'DefaultAxesFontName','Palatino')


%% For paper
linestyle = {'-o','-s','-d','-','-o'};
% linecolor = {'slightly dark blue','slightly dark green','red','white','slightly dark blue'};


%% For presentation macro
% % linestyle = {'-d','-s','-h','*','-o'};
%   linestyle = {'-','-','-','-*','-o'};
% 
% % linecolor = {'slightly dark green','red','light orange','cyan','slightly dark blue'};
%   linecolor = {'white','white','white','cyan','slightly dark blue'};

%% For presentation micro
% % linestyle = {'-d','-s','-h','*','-o'};
%   linestyle = {'-','-s','-','-','-o'};
% 
% % linecolor = {'slightly dark green','red','light orange','cyan','slightly dark blue'};
%   linecolor = {'white','red','white','white','m'};
  
%% For comparison micro macro
% linestyle = {'-','-','-o','-o'};
% linecolor = {'white','white','slightly dark blue','m'};

% linestyle = {'-','-','-','-o'};
% linecolor = {'white','white','white','m'};


%% Code
[nendovar, ~] = size(variables);
comparison = length(models);
first_obs = 1; %0 if you want th eIRF to start from 0 (no shock), 1 otherwise
mkdir('IRF')

%% Find the above variables within the endogenous vector of each model
flag_aux(1:hh,1:tt) = 0; % to keep track of missing variables 
for tt=1:comparison
    for hh=1:nendovar
        aux = strmatch(variables{hh,1},OUT.(models{tt}).M_.endo_names,'exact');
        if isempty(aux)
            select(hh,tt) = 1;
            flag_aux(hh,tt) = NaN; % if a variable is missing, put a NaN
        else
            select(hh,tt) = aux;
        end
    end
end
flag = isnan(flag_aux); % Transform into logical (to be udes later)
  

%% Plot
% if ceil(sqrt(nendovar))>3
%     row = 4;
%     col = 4;
% else
%     row = ceil(sqrt(nendovar));
%     col = ceil(sqrt(nendovar));
% end

row = 2;
col = 4;
NumFig = ceil(nendovar/(row*col));


if exist('legends','var'); FigSize(24,10.5); else FigSize(24,10); end;
mm = 1; % mm indexes the figure number
nn = 1; % nn indexes the variable number
while mm<NumFig+1
    for ii=1:row*col % ii indexes the subplot
        if nn>nendovar
            break
        end
        subflag = subplot(row,col,ii);
        if first_obs==0
            x = [0:IRFsteps];
            xaxis = zeros(1,IRFsteps+1);
        else
            x = [1:IRFsteps];
            xaxis = zeros(1,IRFsteps);
        end
        for jj=1:comparison % jj indexes the model
            aux2 = OUT.(models{jj}).IRF2(:,:);
            aux2 = aux2(select(:,jj),:);
            aux1 = OUT.(models{jj}).IRF1(:,:);
            aux1 = aux1(select(:,jj),:);
            
            aux = log(aux2)-log(aux1);
            % Substitute missing variable with NaN
            if sum(sum(flag))>0
                aux(flag(:,jj),:) = 0;
            end
            rescale = str2double(char(variables(nn,3)));
            if first_obs==0
                h(jj) = plot(x,rescale*100*aux(nn,:),linestyle{jj},'Color',cmap(jj),'LineWidth',1,'MarkerSize',5);
            else
                h(jj) = plot(x,rescale*100*aux(nn,2:end),linestyle{jj},'Color',cmap(jj),'LineWidth',1,'MarkerSize',5);
            end
            hold on
            upp(jj) = max(rescale*100*aux(nn,:));
            low(jj) = min(rescale*100*aux(nn,:));
        end
        plot(x,xaxis,':k','LineWidth',.1)
        ax = gca;
        ax.YAxis.Exponent = 0;
        title(variables(nn,2))
%         opt = FigFontOption(10); opt.title_weight = 'bold'; FigFont(opt)
        if first_obs==0; xlim([0 IRFsteps]); else xlim([1 IRFsteps]); end
        if or(min(low),max(upp)) ~= 0; ylim( [min(low) max(upp)]); end
%         if mod((ii-1)/col,1)==0 || nn==1
%             ylabel('Deviation from SS','FontSize',8)
%         end
        nn = nn + 1;
    end
    junktext = '';
    for jj=1:comparison
        if jj==comparison
            junktext = [junktext '--' models{jj}]; 
        else
            junktext = [junktext '--' models{jj} '--&']; 
        end
    end
    box on
    set(gca, 'Layer','top')
    uistack(gca,'top')
    figure_name = ['IRF\Shock_to_' strtrim(OUT.(models{jj}).M_.exo_names(whichShock,:)) '-' junktext '_' num2str(mm)  ];
    if exist('legends','var'); if mm==NumFig; optleg = LegOption; LegSubplot(legends,optleg); end; end
%     FigFont(opt)
    set(gcf, 'Color', 'w');
    print(figure_name,'-dpdf');
    clf('reset');
    mm=mm+1;
end

close all