clear all
load RESULTS

order = options_.order;
% Number of draws for 
ndraws = 50;
% Steps for IRFs
IRFsteps = 24;
% Steps for simulation
SIMULsteps = 500;
% Number of end variables and number of shocks 
nendovar = size(M_.endo_names,1);
nexovar  = size(M_.exo_names,1);


%% Comparison?
disp(' ')
disp('How many IRFs do you want to compare?:')
comparison = input('? ');

%% Choose which type of IRF you want to plot
disp(' ')
disp('Choose the alternative initial conditions:')
disp('1 --> From a random point')
disp('2 --> From ergodic mean')
disp('3 --> From steady state')
disp('4 --> From a boom')
disp('5 --> From a bust')
for hh=1:comparison
    irf_type(hh) = input('? ');
end
    
%% Define which exogenous process to shock
disp('Which shock ?')
junk(1:nexovar,1)={': '};
disp(' ')
disp([num2str([1:nexovar]') char(junk) M_.exo_names])
whichShock = input('? ');
clear junk


%% Compute IRFs
for hh=1:comparison % loop for comparison
    
    % Initialize
    IRFdraws(1:nendovar,1:IRFsteps+1,1:ndraws) = NaN;
    IRF(1:nendovar,1:IRFsteps+1,1:comparison) = NaN;
    
    % Loop to compute mean response
    for kk=1:ndraws
        % Draw random shocks and simulate the model to get Y
        shock = randn(SIMULsteps,nexovar);
        Y = simult_(oo_.steady_state,oo_.dr,shock,order);
        
        % Discard burn in
        Y = Y(:,.5*SIMULsteps+1:end);
        
        switch irf_type(hh)
            case 1
                initial = Y(:,end);
            case 2
                initial = mean(Y,2);
            case 3
                initial = oo_.steady_state;
            case 4
                select = Y(2,:)==max(Y(2,:));
                initial = Y(:,select);
            case 5
                select = Y(2,:)==min(Y(2,:));
                initial = Y(:,select);
        end

        % Draw random shock1 and simulate the model to get Y1
        shock1 = zeros(IRFsteps,nexovar);
        shock1(:,whichShock) = randn(IRFsteps,1);
        Y1 = simult_(initial,oo_.dr,shock1,order);

        % shock2 is equal to shock1 +1 standard deviation at chosen point  
        shock2 = shock1;
        shock2(1,whichShock) = 1 + shock2(1,whichShock);

        % Simulate the model to get Y2
        Y2 = simult_(initial,oo_.dr,shock2,order);

        % Compute the difference between Y2 and Y1
        IRFdraws(:,:,kk) = Y2-Y1;

    end

    % Average over the number of draws
    IRF(:,:,hh) = mean(IRFdraws,3);
    
end



%% Plot
linestyle = {'-h';'-o';'-d';'-x';'-s'};
linecolor = {'dark blue';'very light blue';'dark green';'green';'cyan'};
subplots  = {'Random';'Ergodic mean';'Steady state';'Boom';'Bust'};

figure

if ceil(sqrt(nendovar))>3
    nplots = 4;
else
    nplots = ceil(sqrt(nendovar));
end
    
NumFig = ceil(nendovar/nplots^2);

FigSize(32,16)
mm = 1; % mm indexes the figure number
nn = 1; % nn indexes the variable number
while mm<NumFig+1
    for ii=1:nplots^2 % ii indexes the subplot
        if nn>nendovar
            break
        end
        subplot(nplots,nplots,ii)
        x = [0:IRFsteps];
        xaxis = zeros(1,IRFsteps+1);
        for jj=1:comparison % jj indexes the comparison
            plot(x,100*IRF(nn,:,jj),linestyle{jj},'Color',rgb(linecolor{jj}),'LineWidth',2);
            hold on
        end
        plot(x,xaxis,'-r','LineWidth',.5)
        xlim([0 IRFsteps])
        title(M_.endo_names(nn,:),'FontWeight','Bold')
        nn = nn + 1;
    end
    figure_name = ['IRF\IRF_' strtrim(M_.exo_names(whichShock,:)) '_' num2str(mm)];
    LegendSubplot(subplots(irf_type),.2)
    set(gcf, 'Color', 'w');
    export_fig(figure_name,'-pdf')
    suptitle(['Impulse Response to ' strtrim(M_.exo_names(whichShock,:))])
    export_fig(figure_name,'-png')
%     print(figure_name,'-dpdf'); print(figure_name,'-deps'); print(figure_name,'-dpng')
    clf('reset');
    mm=mm+1;
end


close all




