% this code computes irf with a trick. I simulate the model just with the
% exogenous process relative to the shocked variable in the IRF. Therefore,
% the system does not explodes and pruning is not needed. This is probably
% not correct. (DSGE_IRF.m computes the same but using all shocks, probably
% the correct thing to do)

% Order of the solution
order = OUT.(models{tt}).options_.order;

% Number of end variables and number of shocks 
OUT.(models{tt}).nendovar = size(OUT.(models{tt}).M_.endo_names,1);
OUT.(models{tt}).nexovar  = size(OUT.(models{tt}).M_.exo_names,1);


%% Comparison?
disp(' ')
disp('Non linear IRFs are different using different starting points.')
disp('How many IRFs do you want to compare?:')
comparison = input('? ');
% comparison = 1;

%% Choose which type of IRF you want to plot
disp(' ')
disp('Choose the alternative initial conditions:')
disp('1 --> From a random point')
disp('2 --> From ergodic mean')
disp('3 --> From steady state')
disp('4 --> From a boom')
disp('5 --> From a bust')
for hh=1:comparison
    irf_type(hh) = input('? ');
%     irf_type(hh) = 3;
end
    
%% Define which exogenous process to shock
disp('Which shock ?')
junk(1:OUT.(models{tt}).nexovar,1)={': '};
disp(' ')
disp([num2str([1:OUT.(models{tt}).nexovar]') char(junk) OUT.(models{tt}).M_.exo_names])
whichShock = input('? ');
% whichShock = 3;
clear junk


%% Compute IRFs
for hh=1:comparison % loop for comparison
    
    %% Initialize
    IRFdraws(1:OUT.(models{tt}).nendovar,1:IRFsteps+1,1:ndraws) = NaN;
    IRF(1:OUT.(models{tt}).nendovar,1:IRFsteps+1,1:comparison) = NaN;
    
    % Loop to compute mean response
    kk = 1; 
    while kk <= ndraws
        disp('---------------------------')
        disp(['Draw #: ' num2str(kk)]) % this is the number of draws to compute the average IRF

        %% Simulation
        if irf_type(hh)~=3
            % Draw random shocks and simulate the model to get Y. If the model
            % is not stable, re-draw the shocks (up to iter_max iterations)
            flag = 0; iter_max = 100; iter = 0;
            while flag==0
                iter = iter + 1;
                shock = zeros(SIMULsteps,OUT.(models{tt}).nexovar);
                shock(:,whichShock) = randn(SIMULsteps,1);
%                 shock = randn(SIMULsteps,OUT.(models{tt}).nexovar);
                Y = simult_(OUT.(models{tt}).oo_.steady_state,OUT.(models{tt}).oo_.dr,shock,order);
                if or( sum(sum(isnan(Y)))==0, sum(sum(Y>10^4))==0 ) % when it explodes it goes either to NaN or to very high number
                    flag = 1;
                end
                if iter == iter_max
                    disp('Max iterations has been hit for Y');
                    disp('The model is not stable, simulation is explosive');
                    return
                end
            end
            disp(['Iteration #: ' num2str(iter)]); % this is the number of iteration to get a stable simulation for Y

            % Discard burn in
            Y = Y(:,.5*SIMULsteps+1:end);
        end
        
        %% Create initial values vector
        % For boom and bust set the variable of interest (ex. GDP is 
        % variable number 8)
        boombust = 14;
        
        switch irf_type(hh)
            case 1
                initial = Y(:,end);
            case 2
                initial = mean(Y,2);
            case 3
                initial = OUT.(models{tt}).oo_.steady_state;
            case 4
                select = Y(boombust,:)==max(Y(boombust,:));
                initial = Y(:,select);
            case 5
                select = Y(boombust,:)==min(Y(boombust,:));
                initial = Y(:,select);
        end

%         [OUT.BGG_RISK.M_.endo_names num2str(initial)]
        
        %% Create the shocks and compute IRFs
        % Draw random shock1 and simulate the model to get Y1
        shock1 = zeros(IRFsteps,OUT.(models{tt}).nexovar);
        shock1(:,whichShock) = randn(IRFsteps,1);
%         global M_ options_
%         M_ = OUT.(models{tt}).M_; M_ = OUT.(models{tt}).options_; 
        Y1 = simult_(initial,OUT.(models{tt}).oo_.dr,shock1,order);

        % shock2 is equal to shock1 +2 standard deviation at chosen point. 
        % Then simulate the model to get Y2  
        shock2 = shock1;
        shock2(1,whichShock) = 2 + shock2(1,whichShock);
        Y2 = simult_(initial,OUT.(models{tt}).oo_.dr,shock2,order);

        % If there NaNs in Y1 or Y2 drop the draw
        % Otherwise IRF is difference between Y2 and Y1
        if or( or( sum(sum(isnan(Y1)))>0, sum(sum(Y1>10^4))>0 ), or( sum(sum(isnan(Y2)))>0, sum(sum(Y2>10^10))>0 ))
            disp(['!WARNING! - IRF not stable. Repeat draw #: ' num2str(kk)])
        else
            IRFdraws(:,:,kk) = Y2-Y1;
            kk = kk+1;
        end
       
        % disp(num2str(sum(sum(isnan(Y)))));
        % disp(num2str(sum(sum(isnan(Y1)))));
        % disp(num2str(sum(sum(isnan(Y2)))));
        
        % Clean
        clear initial Y select
    end

    % Average over the number of draws
    OUT.(models{tt}).IRF(:,:,hh) = mean(IRFdraws,3);
    
    disp(' ')
    disp(['============ End of IRF for comparison ' num2str(hh) ' ============']);
    disp(' ')    
end






