%% Preliminaries
set(0,'DefaultTextFontName','Palatino')
set(0,'DefaultAxesFontName','Palatino')
linestyle = {'-',':','-.','--','-x'};
linecolor = {'dark grey','green','red','dark green','very light blue'};
[nendovar names_flag] = size(variables);
comparison = length(models);
first_obs = 1; %0 if you want th eIRF to start from 0 (no shock), 1 otherwise
mkdir('IRF')

%% Find the above variables within the endogenous vector of each model
flag_aux(1:hh,1:tt) = 0; % to keep track of missing variables 
for tt=1:comparison
    for hh=1:nendovar
        aux = strmatch(variables{hh,1},OUT.(models{tt}).M_.endo_names,'exact');
        if isempty(aux)
            select(hh,tt) = 1;
            flag_aux(hh,tt) = NaN; % if a variable is missing, put a NaN
        else
            select(hh,tt) = aux;
        end
    end
end
flag = isnan(flag_aux); % Transform into logical (to be udes later)
  

%% Plot
% if ceil(sqrt(nendovar))>3
%     row = 4;
%     col = 4;
% else
%     row = ceil(sqrt(nendovar));
%     col = ceil(sqrt(nendovar));
% end

row = 4;
col = 3;
NumFig = ceil(nendovar/(row*col));

FigSize(32,16)
mm = 1; % mm indexes the figure number
nn = 1; % nn indexes the variable number
while mm<NumFig+1
    for ii=1:row*col % ii indexes the subplot
        if nn>nendovar
            break
        end
        subplot(row,col,ii)
        if first_obs==0
            x = [0:IRFsteps];
            xaxis = zeros(1,IRFsteps+1);
        else
            x = [1:IRFsteps];
            xaxis = zeros(1,IRFsteps);
        end
        for jj=1:comparison % jj indexes the model
            aux = OUT.(models{jj}).IRF(:,:);
            aux = aux(select(:,jj),:);
            % Substitute missing variable with NaN
            if sum(sum(flag))>0
                aux(flag(:,jj),:) = 0;
            end
            if first_obs==0
                plot(x,100*aux(nn,:),linestyle{jj},'Color',rgb(linecolor{jj}),'LineWidth',1.5);
            else
                plot(x,100*aux(nn,2:end),linestyle{jj},'Color',rgb(linecolor{jj}),'LineWidth',1.5);
            end
            hold on
            upp(jj) = max(100*aux(nn,:));
            low(jj) = min(100*aux(nn,:));
        end
        plot(x,xaxis,'-k','LineWidth',.5)
        if first_obs==0
            xlim([0 IRFsteps])
        else
            xlim([1 IRFsteps])
        end
        if or(min(low),max(upp)) ~= 0
            ylim( [min(low) max(upp)])
        end
        if names_flag == 1
            title(variables(nn,1),'FontWeight','Bold')
        else
            title(variables(nn,2),'FontWeight','Bold')
        end
        nn = nn + 1;
    end
    figure_name = ['IRF\Compare_' strtrim(OUT.(models{jj}).M_.exo_names(whichShock,:)) '_' num2str(mm)  ];
    LegendSubplot(models,.2,.01,'none',1)
    set(gcf, 'Color', 'w');
    export_fig(figure_name,'-pdf')
    SupTitle(['Model Comparison - Impulse Response to ' strtrim(OUT.(models{jj}).M_.exo_names(whichShock,:))])
    export_fig(figure_name,'-png')
%     print(figure_name,'-dpdf'); print(figure_name,'-dpng')
    clf('reset');
    mm=mm+1;
end

close all