%% Solve model in Dynare++
% Identify the current directory
pathfile = [pwd '\'];

%ACB
filename_dynarepp = 'C:\dynare\4.3.1\dynare++';

% Solving the model up to first order 
command  = [filename_dynarepp,'\dynare++.exe --ord 1',' --sim 0 --per 0 --ss-tol 10e-12 --no-centralize --no-irfs ',pathfile,filename_model,'.mod'];
[status, result] = dos (command);
if status
   error(result)
end
% Results are saved in the file filename_1.mat
movefile([pathfile,filename_model,'.mat'],[pathfile,filename_model,'_1','.mat']);

if order_app > 1
    % Solving the model up to second order 
    command  = [filename_dynarepp,'\dynare++.exe --ord 2',' --sim 0 --per 0 --ss-tol 10e-12 --no-centralize --no-irfs ',pathfile,filename_model,'.mod'];
    [status, result] = dos (command);
    if status
       error(result)
    end
    % Results are saved in the file filename_2.mat
    movefile([pathfile,filename_model,'.mat'],[pathfile,filename_model,'_2','.mat']);
end
if order_app > 2
    % Solving the model up to third order 
    command  = [filename_dynarepp,'\dynare++.exe --ord 3',' --sim 0 --per 0 --ss-tol 10e-12 --no-centralize --no-irfs ',pathfile,filename_model,'.mod'];
    [status, result] = dos (command);
    if status
       error(result)
    end
    % Results are saved in the file filename_3.mat
    movefile([pathfile,filename_model,'.mat'],[pathfile,filename_model,'_3','.mat']);
end

% We also delete the follwing files generated by Dynare++
delete([pathfile,filename_model,'_f','.m']);  %filename_model_f. 
delete([pathfile,filename_model,'_ff','.m']); %filename_model_ff.m
delete([pathfile,filename_model,'.dump']);    %filename_model.dump

%% Unfold Dynare output
% We load and unfold the output from Dynare
[f_ss,fv,fvv,fss,fvvv,fssv,fsss,sigma,nv,nu,nx,nz,dyn_vars,dyn_state_vars,dyn_shocks] ...
    = Dynare_Unfold_Matrices(filename_model,order_app);

%% State-Space
% We set up the alternative state space representation
[g_ss_tmp,gv_tmp,gvv_tmp,gss_tmp,gvvv_tmp,gssv_tmp,gsss_tmp,h_ss,hv,hvv,hss,hvvv,hssv,hsss,ny_tmp,label_y_tmp,label_x] = ...
    StateSpace_form(f_ss,fv,fvv,fss,fvvv,fssv,fsss,sigma,nv,nu,nx,nz,dyn_vars,dyn_state_vars,dyn_shocks);
label_u = dyn_shocks;

%% Variable selection
% We only use the selected variables in g(v)
if strcmp(y_select,'all') ~= 1 
     % Ensure that y_select is a column vector of strings
     [ny1,ny2] = size(y_select);
     if ny2 > ny1
         y_select = y_select';
         ny       = ny2;
     else
         ny       = ny1;
     end
     g_ss    = zeros(ny,1);
     gv      = zeros(ny,nv);
     gvv     = zeros(ny,nv,nv);
     gss     = zeros(ny,1);
     gvvv    = zeros(ny,nv,nv,nv);
     gssv    = zeros(ny,nv);
     gsss    = zeros(ny,1);
     for i=1:ny
        for j=1:ny_tmp
            if strcmp(cellstr(y_select(i,:)),cellstr(label_y_tmp(j,:))) == 1
                label_y(i,:)  = label_y_tmp(j,:);
                g_ss(i,:)     = g_ss_tmp(j,:);
                gv(i,:)       = gv_tmp(j,:);
                gvv(i,:,:)    = gvv_tmp(j,:,:);
                gss(i,:)      = gss_tmp(j,:);                
                gvvv(i,:,:,:) = gvvv_tmp(j,:,:,:);                
                gssv(i,:)     = gssv_tmp(j,:);                
                gsss(i,:)     = gsss_tmp(j,:);                                
            end
        end
    end
else
    ny = ny_tmp;
    label_y  = label_y_tmp;
    g_ss     = g_ss_tmp;
    gv       = gv_tmp;
    gvv      = gvv_tmp;
    gss      = gss_tmp;
    gvvv     = gvvv_tmp;
    gssv     = gssv_tmp;
    gsss     = gsss_tmp;
end
clear g_ss_tmp gv_tmp gvv_tmp gss_tmp gvvv_tmp gssv_tmp gsss_tmp ny_tmp label_y_tmp