% This function uses the pruning scheme to simulate a model solved up 
% to third order in Dynare++. The representation used of the model is 
% y_t   = g(x_t-1,u_t,sig)
% x_t+1 = h(x_t,u_t+1,sig)
% By Martin M. Andreasen

function [y_irf,x_irf] = IRF_Pruning1(g_ss,gv,gvv,gss,gvvv,gssv,gsss,h_ss,hv,hvv,hss,hvvv,hssv,hsss,sigma,nirfsteps,seednr,order_app,...
                         ny,nx,nu,nv,whichShock,x_initial,y_initial)

% We set dimensions
y_irf = zeros(ny,nirfsteps);
x_irf = zeros(nx,nirfsteps);

% We set dimensions for 
y_sim = zeros(ny,nirfsteps);
x_sim = zeros(nx,nirfsteps);

u = zeros(nu,nirfsteps);
shock = randn(1,nirfsteps);
u(whichShock,:) = shock;

% Reexpress some of the matrices for the nonlinear terms
Hvv  = 1/2*reshape(hvv,nx,nv^2);
Hvvv = 1/6*reshape(hvvv,nx,nv^3);
Gvv  = 1/2*reshape(gvv,ny,nv^2);
Gvvv = 1/6*reshape(gvvv,ny,nv^3);

% The value of the perturbation parameter
sig  = 1;

for mm=1:2
    if mm==2
        u(whichShock,1) = u(whichShock,1) + 1;
    end
    if order_app == 1
        xf_ba = zeros(nx,1);
        for t=1:nirfsteps      
            % The controls
            y_sim(:,t) = gv*[xf_ba;u(:,t)];

            % Update of the states
            xf_cu = hv*[xf_ba;u(:,t)];

            % Saving the state
            x_sim(:,t) = xf_cu;        

            % Updating the values of xf_ba
            xf_ba      = xf_cu;
        end
    elseif order_app == 2
        xf_ba = zeros(nx,1);
        xs_ba = zeros(nx,1);    
        for t=1:nirfsteps      
            % The controls
            AA    = kron([xf_ba;u(:,t)],[xf_ba;u(:,t)]);
            y_sim(:,t) = gv*([xf_ba;u(:,t)]+[xs_ba;zeros(nu,1)]) + Gvv*AA + 0.5*gss*sig^2;

            % Update of the states
            xf_cu = hv*[xf_ba;u(:,t)];
            xs_cu = hv*[xs_ba;zeros(nu,1)] + Hvv*AA + 0.5*hss*sig^2;

            % Saving the total value of the states
            x_sim(:,t) = xf_cu + xs_cu;

            % Updating the values of xf_ba and xs_ba
            xf_ba = xf_cu;
            xs_ba = xs_cu;        
        end
    elseif order_app == 3
        xf_ba  = zeros(nx,1);
        xs_ba  = zeros(nx,1);    
        xrd_ba = zeros(nx,1);
%         for t=1:1
%             y_sim(:,t) = y_initial;
%             x_sim(:,t) = x_initial;
%         end
        for t=1:nirfsteps      
            % The controls
            AA = kron([xf_ba;u(:,t)],[xf_ba;u(:,t)]);
            BB = kron([xf_ba;u(:,t)],[xs_ba;zeros(nu,1)]);
            CC = kron(AA,[xf_ba;u(:,t)]);
            y_sim(:,t) = gv*([xf_ba;u(:,t)]+[xs_ba;zeros(nu,1)]+[xrd_ba;zeros(nu,1)]) ...
                       + Gvv*(AA + 2*BB) + Gvvv*CC ...
                       + 0.5*gss*sig^2 + 3/6*gssv*sig^2*[xf_ba;u(:,t)] + 1/6*gsss*sig^3;

            % Update of the states
            xf_cu  = hv*[xf_ba;u(:,t)];
            xs_cu  = hv*[xs_ba;zeros(nu,1)]  +   Hvv*AA + 0.5*hss*sig^2;
            xrd_cu = hv*[xrd_ba;zeros(nu,1)] + 2*Hvv*BB + Hvvv*CC ...
                    + 3/6*hssv*sig^2*[xf_ba;u(:,t)]  + 1/6*hsss*sig^3;        

            % Saving the total value of the states
            x_sim(:,t) = xf_cu + xs_cu + xrd_cu;

            % Updating the values of xf_ba, xs_ba, and xrd_ba
            xf_ba  = xf_cu;
            xs_ba  = xs_cu;        
            xrd_ba = xrd_cu;                
        end
        
        % Save in sim_base and sim_impulse the simulation without and with
        % the unitary shock
        if mm==1
            x_sim_base = x_sim;
            y_sim_base = y_sim;
        elseif mm==2
            x_sim_impulse = x_sim;
            y_sim_impulse = y_sim;
        end
    end
end

x_irf = x_sim_impulse - x_sim_base;
y_irf = y_sim_impulse - y_sim_base;

    

% To reproduce the exact simulated sample path from Simulate_Pruning_LinearInov
%x_sim = x_sim(:,2:end);
%y_sim = y_sim(:,2:end);