% ************************************************************************
% This code peforms pruning on a model solved in Dynare++ and computes
% ergodic mean to be used as initial values for IRF
% ************************************************************************
% Ambrogio Cesa-Bianchi, September 2012

% - filename_dynarerepp : a string indicating where the dynare++ file is 
%                         saved. For instance: filename_dynarepp = 'C:\Dynare++';
% - filename_model:       a string for the name of the mod-file. For instance: 
%                         filename_model = 'SGM';
% - order_app:            the approximation order of the model (must be <= 3) because
%                         Dynare++ deals only with 3rd order max
% - numsim:               the length of the simulated time series from the model and the
%                         length of impulse responses
% - seednr:               a number to initialize the random number generator
% - y_select:             string of control variables. We only report the solution 
%                         and simulate for these variables.
% - delete_all:           1 for deleting all the Mat-files from Dynare++ else zero
% - replications:         number of draws for IRFs


%% OUTPUT
% Open this file for a description of the Matlab Workspace
Output_Description


%% Solve the model in Dynare++
Solve_Model


%% Simulation of the model using the pruning method
[y_sim,x_sim] = Simulate_Pruning(g_ss,gv,gvv,gss,gvvv,gssv,gsss,h_ss,hv,hvv,hss,hvvv,hssv,hsss,sigma,numsim,seednr,order_app,ny,nx,nu,nv);
y_sim = y_sim + repmat(g_ss,1,numsim);
y_sim = y_sim(:,discard:end);

x_sim = x_sim + repmat(h_ss,1,numsim);
x_sim = x_sim(:,discard:end);

xy_sim = [x_sim ; y_sim];


%% Define initial values
x_initial = mean(x_sim,2);
y_initial = mean(y_sim,2);
out.ergo_mean = [x_initial ; y_initial];

pp = 1;
for mm=1:nx
    names{pp,1} = label_x(mm,:);
    pp=pp+1;
end
for mm=1:ny
    names{pp,1} = label_y(mm,:);
    pp=pp+1;
end
out.names = names;
save([filename_model(1:end-2) '_ergodic_mean.mat'],'-struct','out')

%% compute moments

select = {'Y';'C';'I'};
for ii=1:length(select)
    aux(ii,1) = strmatch(select{ii},strtrim(names),'exact');
end

X = log(xy_sim(aux,:));
clear aux

X_T = hpfilter(X,1600);
X_C = X - X_T;

[stats stats_l] = SummStats(100*X_C');


%% Clean 
% We delete the Mat.files created by Dynare
if delete_all == 1
    delete([pathfile,filename_model,'_1.mat']);        %filename_model_1.mat
    if order_app > 1
        delete([pathfile,filename_model,'_2.mat']);    %filename_model_2.mat
    end
    if order_app > 2
        delete([pathfile,filename_model,'_3.mat']);    %filename_model_3.mat
    end
    delete *.jnl
    delete *.asv
end

