%% Preliminaries
mkdir('IRF')
set(0,'DefaultTextFontName','Palatino')
set(0,'DefaultAxesFontName','Palatino')
linestyle = {'-',':','-.','--','-x'};
linecolor = {'dark grey','green','red','dark green','very light blue'};
[nendovar names_flag] = size(variables);
comparison = size(filename_model,1);

ii=1;
for kk=1:nx
    aux(ii,1) = {label_x(kk,:)};
    ii=ii+1;
end
for kk=1:ny
    aux(ii,1) = {label_y(kk,:)};
    ii=ii+1;
end

irf = [x_irf ; y_irf];

vnames = char(aux);
clear aux

%% Find the above variables within the endogenous vector of each model
flag_aux(1:nendovar,1:comparison) = 0; % to keep track of missing variables 
for tt=1:comparison
    for hh=1:nendovar
        aux = strmatch(variables{hh,1},vnames,'exact');
        if isempty(aux)
            select(hh,tt) = 1;
            flag_aux(hh,tt) = NaN; % if a variable is missing, put a NaN
        else
            select(hh,tt) = aux;
        end
    end
end
flag = isnan(flag_aux); % Transform into logical (to be udes later)
  

%% Plot
row = 4;
col = 4;
NumFig = ceil(nendovar/(row*col));

FigSize(32,16)
mm = 1; % mm indexes the figure number
nn = 1; % nn indexes the variable number
while mm<NumFig+1
    for ii=1:row*col % ii indexes the subplot
        if nn>nendovar
            break
        end
        subplot(row,col,ii)
        x = [1:nirfsteps];
        xaxis = zeros(1,nirfsteps);
        for jj=1:comparison % jj indexes the model
            aux = irf(select(:,jj),:);
            % Substitute missing variable with NaN
            if sum(sum(flag))>0
                aux(flag(:,jj),:) = 0;
            end
            plot(x,100*aux(nn,:),linestyle{jj},'Color',rgb(linecolor{jj}),'LineWidth',1.5);
            hold on
            upp(jj) = max(100*aux(nn,:));
            low(jj) = min(100*aux(nn,:));
        end
        plot(x,xaxis,'-k','LineWidth',.5)
        xlim([1 nirfsteps])
        if or(min(low),max(upp)) ~= 0
            ylim( [min(low) max(upp)])
        end
        if names_flag == 1
            title(variables(nn,1),'FontWeight','Bold')
        else
            title(variables(nn,2),'FontWeight','Bold')
        end
        nn = nn + 1;
    end
    figure_name = ['IRF\IRF_' strtrim(label_u(whichShock,:)) '_' num2str(mm)  ];
    LegendSubplot({filename_model},.2,.01,'none',1)
    set(gcf, 'Color', 'w');
    export_fig(figure_name,'-pdf')
    SupTitle(['Model Comparison - Impulse Response to ' strtrim(label_u(whichShock,:)) ])
    export_fig(figure_name,'-png')
%     print(figure_name,'-dpdf'); print(figure_name,'-dpng')
    clf('reset');
    mm=mm+1;
end

close all