% This function sets up an alternative state space representation 
% of the model solved up to third order. That is we transform the solution form
% z_t = f(x_t-1,u_t,sig)
% to 
% y_t   = g(x_t-1,u_t,sig)
% x_t+1 = h(x_t,u_t+1,sig)
% where v_t = [x_t-1,u_t]
% By Martin M. Andreasen. 
function [g_ss,gv,gvv,gss,gvvv,gssv,gsss,h_ss,hv,hvv,hss,hvvv,hssv,hsss,ny,label_y,label_x] = ...
    StateSpace_form(f_ss,fv,fvv,fss,fvvv,fssv,fsss,sigma,nv,nu,nx,nz,dyn_vars,dyn_state_vars,dyn_shocks);

% We setup the function h. The dimensions are
h_ss    = zeros(nx,1); %the steady state level
hv      = zeros(nx,nv);
hvv     = zeros(nx,nv,nv);
hss     = zeros(nx,1);
hvvv    = zeros(nx,nv,nv,nv);
hssv    = zeros(nx,nv);
hsss    = zeros(nx,1);

% This index keeps account of the variables from the f function which are put in the h function 
index_f = zeros(1,nz);

% We construct the h function
for i=1:nx
    for j=1:nz
        if strcmp(cellstr(dyn_state_vars(i,:)),cellstr(dyn_vars(j,:))) == 1
            index_f(1,j)  = 1;
            label_x(i,:)  = dyn_state_vars(i,:);
            h_ss(i,1)     = f_ss(j,1);
            hv(i,:)       = fv(j,:);
            hvv(i,:,:)    = fvv(j,:,:);            
            hss(i,1)      = fss(j,1);
            hvvv(i,:,:,:) = fvvv(j,:,:,:);            
            hssv(i,:)     = fssv(j,:);    
            hsss(i,1)     = fsss(j,1);    
        end
    end
end

% We construct the g function. The dimensions are
ny   = nz-nx;
g_ss = zeros(ny,1); %the steady state level
gv   = zeros(ny,nv);
gvv  = zeros(ny,nv,nv);
gss  = zeros(ny,1);
gvvv = zeros(ny,nv,nv,nv);
gssv = zeros(ny,nv);
gsss = zeros(ny,1);   
i = 0;
for j=1:nz
    if index_f(1,j) == 0
        i = i + 1;
        index_f(1,j) = 1; %To indicate that we have used this position
        g_ss(i,1)     = f_ss(j,1);
        label_y(i,:)  = dyn_vars(j,:);
        gv(i,:)       = fv(j,:);
        gvv(i,:,:)    = fvv(j,:,:);
        gss(i,1)      = fss(j,1);
        gvvv(i,:,:,:) = fvvv(j,:,:,:);
        gssv(i,:)     = fssv(j,:);
        gsss(i,1)     = fsss(j,1);
    end
end